(**
   This is a module for printing in the verbose mode.
   @author Naoki Nishida
   @since 2011/08/23
*)
  
(**
   This boolean flag is used for deciding whether in the verbose mode or not. 
   If [vmode] is [true], then this module performed.
*)
let vmode = ref false;;
    
(**
   This function makes [vmode] [true].
*)
let set_vmode () = vmode := true;;
    
(**
   This function makes [vmode] [false].
*)
let unset_vmode () = vmode := false;;
    
(**
   This provides a prompt used for the verbose printing.
*)
let vprompt = ref "%VERBOSE% ";;
    
(**
   This function sets [vprompt] to the given string.
*)
let set_vprompt s = vprompt := s;;
    
(**
   This function replaces properly intermediate occurrences of [\n] 
   in the given string by the [vprompt].
*)
let replaceNL s0 s =
  let s1 = Util.explode s0 in
  let rec replaceNL0 =
    function
	[] -> []
      |	['\n'] -> ['\n']
      |	[c] -> [c]
      | '\n'::cs -> '\n' :: s1 @ replaceNL0 cs
      |	c::cs -> c :: replaceNL0 cs
  in
  Util.implode (replaceNL0 (Util.explode s));;

(**
   This function prints the given string with [vprompt] if [vmode] is [true].
*)
let vprint n s =
  if !vmode then
    begin
      print_string (!vprompt);
      print_string (Util.string_of_indent n);
      print_string (replaceNL !vprompt s)
    end
  else ();;
    
(**
   This stores [vprompt] temporally.
*)
let tmp_vprompt = ref (!vprompt);;
    
(**
   This prints the beginning of the comment.
*)
let vprint_beginComment s =
  if !vmode 
  then
    begin
      tmp_vprompt := !vprompt;
      vprint 0 s;
      vprompt := (!vprompt)^(Util.string_of_indent 1)
    end
  else ();;
    
(**
   This prints the end of the comment.
*)
let vprint_endComment s =
  if !vmode 
  then
    begin
      vprompt := !tmp_vprompt;
      vprint 0 s
    end
  else ();;
    
(**
   This is an object for printing in debug mode.
*)
class debug_print =
  object
    val mutable dmode = false;
    val dprompt = "%DEBUG% ";
    method set = dmode <- true;
    method unset = dmode <- false;
    method dprint n s =
      if dmode then
	begin
	  print_string dprompt;
	  print_string (Util.string_of_indent n);
	  print_string (replaceNL (dprompt^(Util.string_of_indent n)) s)
	end
      else ();
  end;;
